
#include <limits.h>

#include "operatorswitches.h"
#include "positioner.h"
#include "leds.h"


// Global singletons
OperatorSwitches operatorSwitches ;
Positioner positioner ;
Leds leds ;

//  to continue interrupted move()
Positioner::Direction currentDirection ;

/* Move programs for different modes
  terminate on mode change or on any switch change
  A mode change switches to a different loop*()
*/
void loopModeManual(void) {
  switch (operatorSwitches.getManualDirection()) {
    case OperatorSwitches::directionNone:
      // stop moving... by doing nothing here
      // and release the stepper
      positioner.engageStepper(false) ;
      break;
    case OperatorSwitches::directionLeft:
      // drive left ... terminated on switch change
      currentDirection = Positioner::directionLeft ; // "auto" continues in same direction
      positioner.move(Positioner::directionLeft, 0, Positioner::speedSlow) ;
      break;
    case OperatorSwitches::directionRight:
      // drive right ... terminated on switch change
      currentDirection = Positioner::directionRight ;
      positioner.move(Positioner::directionRight, 0, Positioner::speedSlow) ;
      break;
  }
}

/*
   oscillate between end switches. Each move is stopped on switch.
   if end switch: change direction, else continue
*/
void loopModeAuto( enum Positioner::Speed speed) {
  if (positioner.atLeft())
    currentDirection = Positioner::directionRight ;
  else if (positioner.atRight())
    currentDirection = Positioner::directionLeft ;
  else if (currentDirection == Positioner::directionNone)
    // start movement after leaving modeManual
    currentDirection = Positioner::directionLeft ;

  // distance=0: endlessly until any switch change (user&end)
  positioner.move(currentDirection, 0, speed) ;
}

void setup() {
  leds.setup() ;
  operatorSwitches.setup() ;
  positioner.setup() ;
  currentDirection = Positioner::directionNone ;
}

void loop() {

  // find end switch
  //      positioner.calibrate() ;

  /*
      positioner.move(Positioner::directionLeft, 100, Positioner::speedSlow) ;
      positioner.move(Positioner::directionRight, 100, Positioner::speedSlow) ;
  */
  // execute one of the 3 mode programs.
  switch (operatorSwitches.getMode()) {
    case OperatorSwitches::modeManual:
      loopModeManual() ;
      break ;
    case OperatorSwitches::modeSlow:
      loopModeAuto(Positioner::speedSlow) ;
      break ;
    case OperatorSwitches::modeFast:
      loopModeAuto(Positioner::speedFast) ;
      break ;
  }
}
