#ifndef OPERATORSWITCHES_H
#define OPERATORSWITCHES_H

// handles the "mode" and "speed" switches
class OperatorSwitches {
  private:
    // raw GPIOs
    static const int pinModeSlow = 7 ; // red
    static const int pinModeFast = 8 ; // orange
    // do not waste PWM pins here
    static const int pinManualLeft = 12 ; // red
    static const int pinManualRight = 13 ; // orange
    // input 13 shared with internal LED

    // to detect changes
    unsigned lastState ;

  public:
    void setup(void) {
      pinMode(pinModeSlow, INPUT_PULLUP);
      pinMode(pinModeFast, INPUT_PULLUP);
      pinMode(pinManualLeft, INPUT_PULLUP);
      pinMode(pinManualRight, INPUT_PULLUP);
      lastState = 0 ;
      anyChange() ; // update lastState
    }


    /* anyChange()
        monitor all switches for change.
        false if nothing happened.
    */
    bool anyChange() {
      // encode all switches binary
      unsigned newState ;
      newState = (!digitalRead(pinModeSlow)) << pinModeSlow ;
      newState |= (!digitalRead(pinModeFast)) << pinModeFast ;
      newState |= (!digitalRead(pinManualLeft)) << pinManualLeft ;
      newState |= (!digitalRead(pinManualRight)) << pinManualRight ;

      if (lastState == newState)
        return false ;
      else {
        lastState = newState ;
        return true ;
      }
    }

    enum Modes { modeManual, modeSlow, modeFast } ;

    enum Modes getMode() {
      // only 2 switch positions "slow" and "fast" connected
      if (digitalRead(pinModeSlow) == 0)
        return modeSlow ;
      else if (digitalRead(pinModeFast) == 0)
        return modeFast ;
      else
        return modeManual ;
    }

    enum ManualDirection { directionNone, directionLeft, directionRight } ;

    enum ManualDirection getManualDirection() {
      // only 2 switch positions "left" and "right" connected
      if (digitalRead(pinManualLeft) == 0)
        return directionLeft ;
      else if (digitalRead(pinManualRight) == 0)
        return directionRight ;
      else
        return directionNone ;

    }
} ;


#endif // OPERATORSWITCHES_H
